"""
Author: Sebastian Quiles
Based on:
https://stackoverflow.com/questions/33767627/python-write-unittest-for-console-print/46307456#46307456
"""
import io
import sys
from enum import Enum
from typing import Tuple


class _AssertType(Enum):
    """internal enum Types of assertions"""
    EQ = "Equals"
    IN = "In"
    REGEX = "Regex"


class _AssertStdoutContext:
    """"internal Context Class to use it in a with context block"""

    def __init__(self, test_case, assert_type: _AssertType, output_expected: Tuple[str] | str):
        """constructor, testcase is the TestCase original instance, output_expected is the expected
        output on quiting the context"""
        self.test_case = test_case
        self.expected = output_expected
        self.captured = io.StringIO()
        self.assert_type = assert_type

    def __enter__(self):
        """"when the context begin, this method will hook stdout"""
        sys.stdout = self.captured
        return self

    def __exit__(self, exception_type, exception_value, exception_traceback):
        """"where the context finishes, stdout will be unhooked, and final assert is performed"""
        sys.stdout = sys.__stdout__
        captured = self.captured.getvalue()
        if self.assert_type == _AssertType.EQ:
            self.test_case.assertEqual(captured, self.expected)
        elif self.assert_type == _AssertType.IN:
            # pylint: disable=expression-not-assigned
            [self.test_case.assertIn(expected_text, captured) for expected_text in self.expected]
            # pylint: enable=expression-not-assigned
        elif self.assert_type == _AssertType.REGEX:
            self.test_case.assertRegex(captured, self.expected)


class TestCaseStdoutMixin:
    """Class to use as a Mixin Class along django.test.Testcase or unitest.Testcase"""

    def assertStdout(self, expected_output: str):  # noqa: N802 # pylint: disable=invalid-name
        """"method to test if the expected_output is exactly what was streamed in stdout"""
        return _AssertStdoutContext(self, _AssertType.EQ, expected_output)

    def assertStdoutPrints(self, *expected_output: Tuple[str] | str):  # noqa: N802 # pylint: disable=invalid-name
        """"method to test if a list of strings, (*expected_output) is exactly what was
        streamed in stdout separated with /n"""
        expected_output = "\n".join(expected_output) + "\n"
        return _AssertStdoutContext(self, _AssertType.EQ, expected_output)

    def assertStdoutContains(self, *expected_output: Tuple[str] | str):  # noqa: N802 # pylint: disable=invalid-name
        """"method to test if a list of expected_output was printed in stdout"""
        return _AssertStdoutContext(self, _AssertType.IN, *expected_output)

    def assertStdoutRegex(self, expected_regex_output: str):  # noqa: N802 # pylint: disable=invalid-name
        """"method to test if expected_regex_output match with stdout"""
        return _AssertStdoutContext(self, _AssertType.REGEX, expected_regex_output)



import sys

from karpyncho.stdout_context import TestCaseStdoutMixin
from unittest import TestCase


class TestPrint(TestCase, TestCaseStdoutMixin):
    def test_assert_stdout(self):
        with self.assertStdout("test\n"):
            print("test")

    def test_assert_stdout_fails(self):
        with self.assertRaises(AssertionError):
            with self.assertStdout("test\n"):
                print("hello world")

    def test_assert_stdout_prints(self):
        with self.assertStdoutPrints("test1", "test2", "test3"):
            print("test1")
            print("test2")
            pass
            print("test3")

    def test_assert_stdout_prints_fails(self):
        with self.assertRaises(AssertionError):
            with self.assertStdoutPrints("test1", "no test", "test3"):
                print("test1")
                print("test2")
                pass
                print("test3")

    def test_assert_stdout_using_write(self):
        with self.assertStdout("test\n"):
            sys.stdout.write("test\n")

    def test_assert_stdout_using_write_fails(self):
        with self.assertRaises(AssertionError):
            with self.assertStdout("will fail\n"):
                sys.stdout.write("test\n")

    def test_assert_stdout_not_stderr(self):
        with self.assertStdout(""):
            sys.stderr.write("test\n")

    def test_assert_stdout_contains_one_expected(self):
        with self.assertStdoutContains("xxxx"):
            print("test1")
            print("xxxx")
            print("test3")

    def test_assert_stdout_contains_one_expected_fails(self):
        with self.assertRaises(AssertionError):
            with self.assertStdoutContains("yyyyy"):
                print("test1")
                print("xxxx")
                print("test3")

    def test_assert_stdout_contains_one_line(self):
        with self.assertStdoutContains("mytest"):
            print("test1 mytest test2")

    def test_assert_stdout_contains_one_line_fails(self):
        with self.assertRaises(AssertionError):
            with self.assertStdoutContains("blah blah"):
                print("test1 mytest test2")

    def test_assert_stdout_contains_several_expected(self):
        with self.assertStdoutContains("mytest",  "test1", "world"):
            print("hello world")
            print("test1 mytest test2")
            print("goodbye world")

    def test_assert_stdout_contains_several_expected_fails(self):
        with self.assertRaises(AssertionError):
            with self.assertStdoutContains("mytest", "blah blah"):
                print("hello world")
                print("test1 mytest test2")
                print("goodbye world")

    def test_assert_stdout_regex(self):
        with self.assertStdoutRegex(r'^a+b+c+$'):
            print("aaaaaaaaaaaaaaaaaabbbbbbbbbbbbbbbbbbccccccccccccc")

    def test_assert_stdout_regex_fails(self):
        with self.assertRaises(AssertionError):
            with self.assertStdoutRegex(r'^a+b+c+$'):
                print("dddddaaaaaaaaaaaaaaaaaabbbbbbbbbbbbbbbbbbccccccccccccc")
